@@ -12,6 +12,7 @@ module Network.HTTP.Client.Manager
     , getConn
     , failedConnectionException
     , defaultManagerSettings
+    , rawConnectionModifySocket
     ) where
 
 #if !MIN_VERSION_base(4,6,0)
@@ -31,7 +32,7 @@ import Data.Text (Text)
 import qualified Data.Text as T
 
 import Control.Monad.IO.Class (MonadIO, liftIO)
-import Control.Monad (unless)
+import Control.Monad (unless, join)
 import Control.Exception (mask_, SomeException, bracket, catch, throwIO, fromException, mask, IOException, Exception (..), handle)
 import Control.Concurrent (forkIO, threadDelay)
 import Data.Time (UTCTime (..), Day (..), DiffTime, getCurrentTime, addUTCTime)
@@ -47,15 +48,24 @@ import Network.HTTP.Client.Types
 import Network.HTTP.Client.Connection
 import Network.HTTP.Client.Headers (parseStatusHeaders)
 
+-- | A value for the @managerRawConnection@ setting, but also allows you to
+-- modify the underlying @Socket@ to set additional settings. For a motivating
+-- use case, see: <https://github.com/snoyberg/http-client/issues/71>.
+--
+-- Since 0.3.8
+rawConnectionModifySocket :: (NS.Socket -> IO ())
+                          -> IO (Maybe NS.HostAddress -> String -> Int -> IO Connection)
+rawConnectionModifySocket = return . openSocketConnection
+
 -- | Default value for @ManagerSettings@.
 --
 -- Since 0.1.0
 defaultManagerSettings :: ManagerSettings
 defaultManagerSettings = ManagerSettings
     { managerConnCount = 10
-    , managerRawConnection = return openSocketConnection
+    , managerRawConnection = return $ openSocketConnection (const $ return ())
     , managerTlsConnection = return $ \_ _ _ -> throwIO TlsNotSupported
-    , managerTlsProxyConnection = return $ \_ _ _ _ _ -> throwIO TlsNotSupported
+    , managerTlsProxyConnection = return $ \_ _ _ _ _ _ -> throwIO TlsNotSupported
     , managerResponseTimeout = Just 30000000
     , managerRetryableException = \e ->
         case fromException e of
@@ -76,32 +86,42 @@ defaultManagerSettings = ManagerSettings
                     Just e -> toException $ InternalIOException e
                     Nothing -> se
          in handle $ throwIO . wrapper
+    , managerIdleConnectionCount = 512
     }
 
 takeSocket :: Manager -> ConnKey -> IO (Maybe Connection)
 takeSocket man key =
     I.atomicModifyIORef (mConns man) go
   where
-    go Nothing = (Nothing, Nothing)
-    go (Just m) =
+    go ManagerClosed = (ManagerClosed, Nothing)
+    go mcOrig@(ManagerOpen idleCount m) =
         case Map.lookup key m of
-            Nothing -> (Just m, Nothing)
-            Just (One a _) -> (Just $ Map.delete key m, Just a)
-            Just (Cons a _ _ rest) -> (Just $ Map.insert key rest m, Just a)
+            Nothing -> (mcOrig, Nothing)
+            Just (One a _) ->
+                let mc = ManagerOpen (idleCount - 1) (Map.delete key m)
+                 in mc `seq` (mc, Just a)
+            Just (Cons a _ _ rest) ->
+                let mc = ManagerOpen (idleCount - 1) (Map.insert key rest m)
+                 in mc `seq` (mc, Just a)
 
 putSocket :: Manager -> ConnKey -> Connection -> IO ()
 putSocket man key ci = do
     now <- getCurrentTime
-    msock <- I.atomicModifyIORef (mConns man) (go now)
-    maybe (return ()) connectionClose msock
+    join $ I.atomicModifyIORef (mConns man) (go now)
   where
-    go _ Nothing = (Nothing, Just ci)
-    go now (Just m) =
-        case Map.lookup key m of
-            Nothing -> (Just $ Map.insert key (One ci now) m, Nothing)
+    go _ ManagerClosed = (ManagerClosed , connectionClose ci)
+    go now mc@(ManagerOpen idleCount m)
+        | idleCount >= mIdleConnectionCount man = (mc, connectionClose ci)
+        | otherwise = case Map.lookup key m of
+            Nothing ->
+                let cnt' = idleCount + 1
+                    m' = ManagerOpen cnt' (Map.insert key (One ci now) m)
+                 in m' `seq` (m', return ())
             Just l ->
                 let (l', mx) = addToList now (mMaxConns man) ci l
-                 in (Just $ Map.insert key l' m, mx)
+                    cnt' = idleCount + maybe 0 (const 1) mx
+                    m' = ManagerOpen cnt' (Map.insert key l' m)
+                 in m' `seq` (m', maybe (return ()) connectionClose mx)
 
 -- | Add a new element to the list, up to the given maximum number. If we're
 -- already at the maximum, return the new value as leftover.
@@ -128,7 +148,7 @@ newManager ms = do
     rawConnection <- managerRawConnection ms
     tlsConnection <- managerTlsConnection ms
     tlsProxyConnection <- managerTlsProxyConnection ms
-    mapRef <- I.newIORef (Just Map.empty)
+    mapRef <- I.newIORef $! ManagerOpen 0 Map.empty
     wmapRef <- I.mkWeakIORef mapRef $ closeManager' mapRef
     _ <- forkIO $ reap wmapRef
     let manager = Manager
@@ -140,12 +160,12 @@ newManager ms = do
             , mTlsProxyConnection = tlsProxyConnection
             , mRetryableException = managerRetryableException ms
             , mWrapIOException = managerWrapIOException ms
+            , mIdleConnectionCount = managerIdleConnectionCount ms
             }
     return manager
 
 -- | Collect and destroy any stale connections.
-reap :: Weak (I.IORef (Maybe (Map.Map ConnKey (NonEmptyList Connection))))
-     -> IO ()
+reap :: Weak (I.IORef ConnsMap) -> IO ()
 reap wmapRef =
     mask_ loop
   where
@@ -159,16 +179,13 @@ reap wmapRef =
     goMapRef mapRef = do
         now <- getCurrentTime
         let isNotStale time = 30 `addUTCTime` time >= now
-        mtoDestroy <- I.atomicModifyIORef mapRef (findStaleWrap isNotStale)
-        case mtoDestroy of
-            Nothing -> return () -- manager is closed
-            Just toDestroy -> do
-                mapM_ safeConnClose toDestroy
-                loop
-    findStaleWrap _ Nothing = (Nothing, Nothing)
-    findStaleWrap isNotStale (Just m) =
+        toDestroy <- I.atomicModifyIORef mapRef (findStaleWrap isNotStale)
+        mapM_ safeConnClose toDestroy
+        loop
+    findStaleWrap _ ManagerClosed = (ManagerClosed, [])
+    findStaleWrap isNotStale (ManagerOpen idleCount m) =
         let (x, y) = findStale isNotStale m
-         in (Just x, Just y)
+         in (ManagerOpen (idleCount - length y) x, y)
     findStale isNotStale =
         findStale' id id . Map.toList
       where
@@ -244,11 +261,13 @@ neFromList xs =
 closeManager :: Manager -> IO ()
 closeManager = closeManager' . mConns
 
-closeManager' :: I.IORef (Maybe (Map.Map ConnKey (NonEmptyList Connection)))
+closeManager' :: I.IORef ConnsMap
               -> IO ()
 closeManager' connsRef = mask_ $ do
-    m <- I.atomicModifyIORef connsRef $ \x -> (Nothing, x)
-    mapM_ (nonEmptyMapM_ safeConnClose) $ maybe [] Map.elems m
+    !m <- I.atomicModifyIORef connsRef $ \x -> (ManagerClosed, x)
+    case m of
+        ManagerClosed -> return ()
+        ManagerOpen _ m -> mapM_ (nonEmptyMapM_ safeConnClose) $ Map.elems m
 
 -- | Create, use and close a 'Manager'.
 --
@@ -361,4 +380,4 @@ getConn req m
                         sh@(StatusHeaders status _ _) <- parseStatusHeaders conn
                         unless (status == status200) $
                             throwIO $ ProxyConnectException ultHost ultPort $ Left $ S8.pack $ show sh
-                 in mTlsProxyConnection m connstr parse
+                 in mTlsProxyConnection m connstr parse (S8.unpack ultHost)
