@@ -19,6 +19,7 @@ module Network.HTTP.Client.Request
     , needsGunzip
     , requestBuilder
     , useDefaultTimeout
+    , setQueryString
     ) where
 
 import Data.Maybe (fromMaybe, isJust)
@@ -30,7 +31,7 @@ import Numeric (showHex)
 
 import Data.Default.Class (Default (def))
 
-import Blaze.ByteString.Builder (Builder, fromByteString, fromLazyByteString, toByteStringIO)
+import Blaze.ByteString.Builder (Builder, fromByteString, fromLazyByteString, toByteStringIO, flush)
 import Blaze.ByteString.Builder.Char8 (fromChar)
 
 import qualified Data.ByteString as S
@@ -290,8 +291,11 @@ requestBuilder req Connection {..} =
             RequestBodyLBS lbs -> (Just $ L.length lbs, writeBuilder $ builder `mappend` fromLazyByteString lbs)
             RequestBodyBS bs -> (Just $ fromIntegral $ S.length bs, writeBuilder $ builder `mappend` fromByteString bs)
             RequestBodyBuilder i b -> (Just $ i, writeBuilder $ builder `mappend` b)
-            RequestBodyStream i stream -> (Just i, writeBuilder builder >> writeStream False stream)
-            RequestBodyStreamChunked stream -> (Nothing, writeBuilder builder >> writeStream True stream)
+
+            -- See https://github.com/snoyberg/http-client/issues/74 for usage
+            -- of flush here.
+            RequestBodyStream i stream -> (Just i, writeBuilder (builder `mappend` flush) >> writeStream False stream)
+            RequestBodyStreamChunked stream -> (Nothing, writeBuilder (builder `mappend` flush) >> writeStream True stream)
 
     writeStream isChunked withStream =
         withStream loop
@@ -366,3 +370,9 @@ requestBuilder req Connection {..} =
         <> fromByteString ": "
         <> fromByteString v
         <> fromByteString "\r\n"
+
+-- | Set the query string to the given key/value pairs.
+--
+-- Since 0.3.6
+setQueryString :: [(S.ByteString, Maybe S.ByteString)] -> Request -> Request
+setQueryString qs req = req { queryString = W.renderQuery True qs }
