@@ -8,31 +8,24 @@ module Network.HTTP.Client.Response
     , lbsResponse
     ) where
 
-import Control.Arrow (first)
-import Control.Monad (liftM)
+import Control.Monad ((>=>))
 
 import Control.Exception (throwIO)
-import Control.Monad.IO.Class (MonadIO, liftIO)
 
-import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 
-import qualified Data.CaseInsensitive as CI
-
-import Data.Default (def)
+import Data.Default.Class (def)
 
 import qualified Network.HTTP.Types as W
 import Network.URI (parseURIReference)
 
 import Network.HTTP.Client.Types
 
-import Network.HTTP.Client.Manager
 import Network.HTTP.Client.Request
 import Network.HTTP.Client.Util
 import Network.HTTP.Client.Body
 import Network.HTTP.Client.Headers
-import Network.HTTP.Client.Connection
 
 import System.Timeout (timeout)
 
@@ -73,6 +66,7 @@ getRedirectedRequest req hs cookie_jar code
                     { method = "GET"
                     , requestBody = RequestBodyBS ""
                     , cookieJar = cookie_jar'
+                    , requestHeaders = filter ((/= W.hContentType) . fst) $ requestHeaders req'
                     }
                 else req' {cookieJar = cookie_jar'}
     | otherwise = Nothing
@@ -97,11 +91,7 @@ getResponse connRelease timeout'' req@(Request {..}) conn = do
     let timeout' =
             case timeout'' of
                 Nothing -> id
-                Just useconds -> \ma -> do
-                    x <- timeout useconds ma
-                    case x of
-                        Nothing -> liftIO $ throwIO ResponseTimeout
-                        Just y -> return y
+                Just t -> timeout t >=> maybe (throwIO ResponseTimeout) return
     StatusHeaders s version hs <- timeout' $ parseStatusHeaders conn
     let mcl = lookup "content-length" hs >>= readDec . S8.unpack
 
