@@ -22,6 +22,7 @@ module Network.HTTP.Client.Types
     , Response (..)
     , ResponseClose (..)
     , Manager (..)
+    , ConnsMap (..)
     , ManagerSettings (..)
     , NonEmptyList (..)
     , ConnHost (..)
@@ -365,7 +366,7 @@ data Request = Request
     , responseTimeout :: Maybe Int
     -- ^ Number of microseconds to wait for a response. If
     -- @Nothing@, will wait indefinitely. Default: use
-    -- 'managerResponseTimeout' (which by default is 5 seconds).
+    -- 'managerResponseTimeout' (which by default is 30 seconds).
     --
     -- Since 0.1.0
     , getConnectionWrapper :: Maybe Int
@@ -458,7 +459,7 @@ data ManagerSettings = ManagerSettings
       -- ^ Create a TLS connection. Default behavior: throw an exception that TLS is not supported.
       --
       -- Since 0.1.0
-    , managerTlsProxyConnection :: IO (S.ByteString -> (Connection -> IO ()) -> Maybe NS.HostAddress -> String -> Int -> IO Connection)
+    , managerTlsProxyConnection :: IO (S.ByteString -> (Connection -> IO ()) -> String -> Maybe NS.HostAddress -> String -> Int -> IO Connection)
       -- ^ Create a TLS proxy connection. Default behavior: throw an exception that TLS is not supported.
       --
       -- Since 0.2.2
@@ -466,7 +467,7 @@ data ManagerSettings = ManagerSettings
       -- ^ Default timeout (in microseconds) to be applied to requests which do
       -- not provide a timeout value.
       --
-      -- Default is 5 seconds
+      -- Default is 30 seconds
       --
       -- Since 0.1.0
     , managerRetryableException :: SomeException -> Bool
@@ -483,6 +484,16 @@ data ManagerSettings = ManagerSettings
     -- Default: wrap all @IOException@s in the @InternalIOException@ constructor.
     --
     -- Since 0.1.0
+    , managerIdleConnectionCount :: Int
+    -- ^ Total number of idle connection to keep open at a given time.
+    --
+    -- This limit helps deal with the case where you are making a large number
+    -- of connections to different hosts. Without this limit, you could run out
+    -- of file descriptors.
+    --
+    -- Default: 512
+    --
+    -- Since 0.3.7
     }
     deriving T.Typeable
 
@@ -492,7 +503,7 @@ data ManagerSettings = ManagerSettings
 --
 -- Since 0.1.0
 data Manager = Manager
-    { mConns :: I.IORef (Maybe (Map.Map ConnKey (NonEmptyList Connection)))
+    { mConns :: I.IORef ConnsMap
     -- ^ @Nothing@ indicates that the manager is closed.
     , mMaxConns :: Int
     -- ^ This is a per-@ConnKey@ value.
@@ -500,12 +511,17 @@ data Manager = Manager
     -- ^ Copied from 'managerResponseTimeout'
     , mRawConnection :: Maybe NS.HostAddress -> String -> Int -> IO Connection
     , mTlsConnection :: Maybe NS.HostAddress -> String -> Int -> IO Connection
-    , mTlsProxyConnection :: S.ByteString -> (Connection -> IO ()) -> Maybe NS.HostAddress -> String -> Int -> IO Connection
+    , mTlsProxyConnection :: S.ByteString -> (Connection -> IO ()) -> String -> Maybe NS.HostAddress -> String -> Int -> IO Connection
     , mRetryableException :: SomeException -> Bool
     , mWrapIOException :: forall a. IO a -> IO a
+    , mIdleConnectionCount :: Int
     }
     deriving T.Typeable
 
+data ConnsMap
+    = ManagerClosed
+    | ManagerOpen {-# UNPACK #-} !Int !(Map.Map ConnKey (NonEmptyList Connection))
+
 data NonEmptyList a =
     One a UTCTime |
     Cons a Int UTCTime (NonEmptyList a)
